/*
 * File: SeeTheUS.java
 * ===============================================================
 * A program to visualize the United States based on US Census
 * records from 2000.
 */

import acm.program.*;
import acm.graphics.*;
import java.awt.*;
import java.io.*;

public class SeeTheUS extends GraphicsProgram {
	/* Make the window large so that we can see more detail. */
	public static final int APPLICATION_WIDTH = 1000;
	public static final int APPLICATION_HEIGHT = 750;
	
	/* The viewpoint coordinates - the minimum and maximum longitude
	 * and latitude.
	 */
	private static final double MIN_LONGITUDE = -180;
	private static final double MAX_LONGITUDE = -60;
	
	private static final double MIN_LATITUDE = +15;
	private static final double MAX_LATITUDE = +75;
	
	/* The name of the cities file. */
	private static final String CITIES_FILE = "us-cities.txt";
	
	public void run() {
		try {
			/* Open the cities file for reading. */
			BufferedReader br = new BufferedReader(new FileReader(CITIES_FILE));
			
			while (true) {
				/* Read the next entry, which consists of the city, the latitude,
				 * and the longitude.
				 */
				String city = br.readLine();
				String latitude = br.readLine();
				String longitude = br.readLine();
				
				/* If no more data is left, stop. */
				if (city == null || latitude == null || longitude == null) {
					break;
				}
				
				/* Convert the longitude and latitude to x and y coordinates on the screen.
				 * The calls to Double.parseDouble are necessary here to convert the
				 * string representations of the doubles to actual doubles.
				 */
				double x = longitudeToXCoordinate(Double.parseDouble(longitude));
				double y = latitudeToYCoordinate(Double.parseDouble(latitude));
				
				/* Draw the city. */
				plotPixel(x, y);
			}
			
			br.close();			
		} catch (IOException e) {
			/* TODO: Actual error handling? */
			println(":-(");
		}
	}
	
	/**
	 * Plots a pixel at the specified (x, y) coordinate.
	 * 
	 * @param x The X coordinate.
	 * @param y The Y coordinate.
	 */
	private void plotPixel(double x, double y) {
		/* Create a 1x1 pixel of the given color. */
		GRect pixel = new GRect(x, y, 1, 1);
		pixel.setFilled(true);
		pixel.setColor(Color.BLACK);
		add(pixel);
	}
	
	/**
	 * Given a raw longitude, returns the screen x coordinate where
	 * it should be displayed.
	 * 
	 * @param longitude The longitude in question.
	 * @return Where it maps to as an x coordinate.
	 */
	private double longitudeToXCoordinate(double longitude) {
		return getWidth() * (longitude - MIN_LONGITUDE) / (MAX_LONGITUDE - MIN_LONGITUDE); 
	}
	
	/**
	 * Given a raw latitude, returns the screen y coordinate where
	 * it should be displayed.
	 * 
	 * @param latitude The latitude in question.
	 * @return Where it maps to as a y coordinate.
	 */
	private double latitudeToYCoordinate(double latitude) {
		return getHeight() * (1.0 - (latitude - MIN_LATITUDE) / (MAX_LATITUDE - MIN_LATITUDE)); 
	}
}
